<?php
  
function wp_sm_modify_pages_table( $column ) {
    $column['is_protected'] = 'Is Protected';

    return $column;
}

function wp_sm_modify_pages_table_row( $column_name, $page_id ) {

    switch ($column_name) {
        case 'is_protected' :
          $protected = wp_sm_object_is_protected($page_id, "page");
          echo ($protected ? __('Protected', 'wp-sm') : '');
          if(wp_sm_object_is_protected_with_inheritance($page_id, "page")){
            echo ' ('.__('Inherited', 'wp-sm').')';
          }
          break;

        default:
    }
}

// Object is not directly protected, but inheritedly protected
function wp_sm_object_is_protected_with_inheritance($object_id, $object_class){
  $inherited_protected = false;
  
  if(wp_sm_no_protection()){
    return false;
  }
  
  switch ($object_class) {
    case 'page' :
      if(wp_sm_object_is_protected($object_id, $object_class) && !get_post_meta($object_id, "wp_sm_protected", true)){
        $inherited_protected = true;
      }
      
      break;

    default:
  }
  
  return $inherited_protected;
  
}

function wp_sm_object_is_protected($object_id, $object_class){
  $protected = false;
  
  if(wp_sm_no_protection()){
    return false;
  }
  
  switch ($object_class) {
    case 'page' :
      $protected = get_post_meta($object_id, "wp_sm_protected", true);
      
      if(!$protected){
        // TODO: check permission on parent pages
        $ancestors = get_ancestors( $object_id, $object_class );
        if($ancestors && count($ancestors) > 0){
          foreach ( $ancestors as $ancestor ) {
            if(get_post_meta($ancestor, "wp_sm_protected", true) && get_post_meta($ancestor, "wp_sm_protect_childs", true) ){
              $protected = true;
              break;
            }
          }
        }
      }
      
      break;

    default:
  }
  
  return $protected;
  
}

function wp_sm_user_has_access($object_id, $object_class, $user_id){
  if( !wp_sm_object_is_protected($object_id, $object_class) || (!($user_id instanceof WP_Error) && user_can($user_id, 'manage_options')) || wp_sm_no_protection()){
    return true;
  } else {
    if ($user_id instanceof WP_Error){
      //die("Error:" . $user_id->get_error_message());
      return false;
    }

    $permission = Permission::find("first", array("conditions" => array("user_id = ? AND object_id = ? AND object_class = ?",$user_id, $object_id, $object_class)));
    
    if($permission){
      return true;
    } else {
      $ancestors = get_ancestors( $object_id, $object_class );
      if($ancestors && count($ancestors) > 0){
        foreach ( $ancestors as $ancestor ) {
          if( get_post_meta($ancestor, "wp_sm_protect_childs", true) ){
            if(Permission::find("first", array("conditions" => array("user_id = ? AND object_id = ? AND object_class = ?",$user_id, $ancestor, $object_class)))){
              return true;
            }
          }
        }
      }
    }
    
  }
  
  return false;
  
}

function wp_sm_filter_protected_pages($content){
  global $wpdb, $current_user, $post;
  wp_get_current_user();
  
  if(!wp_sm_prot_use_redirect() && !wp_sm_no_protection()){
    $user_id = $current_user->ID;

    if(! wp_sm_user_has_access($post->ID, 'page', $user_id)){
      $content = __("Non hai accesso a questo contenuto", 'wp-sm');
      remove_filter('the_content', 'wp_sm_filter_protected_pages');
      add_filter('the_content', 'wp_sm_void_filter_protected_pages');
    }
  } 
  
  return do_shortcode($content);
}

function wp_sm_filter_protected_pages_redirect($args){
  global $wpdb, $current_user, $post;
  wp_get_current_user();

  $redir_url = wp_sm_custom_redirect_url_for($post->ID);
  if ($redir_url == false || $redir_url == "" ){
    $redir_url = wp_sm_prot_use_redirect();
  }
  
  if($redir_url){
    $user_id = $current_user->ID;

    if(! wp_sm_user_has_access($post->ID, 'page', $user_id)){
      
      wp_redirect( $redir_url ); exit;
    }
  }
  
}

function wp_sm_custom_redirect_url_for($page_id){
  $val = get_post_meta($page_id, 'sm_custom_redirect', true);
  return $val;
}


function wp_sm_void_filter_protected_pages($content){
  return '';  
}

# Set the rewrite rule for autologin

// Register a URL that will set this variable to true
function wp_sm_autolog_init() {
    add_rewrite_rule( '^wp_sm_autolog$', 'index.php?wp_sm_autolog=true', 'top' );
}

// But WordPress has a whitelist of variables it allows, so we must put it on that list
function wp_sm_autolog_query_vars( $query_vars )
{
    $query_vars[] = 'wp_sm_autolog';
    return $query_vars;
}

// If this is done, we can access it later
// This example checks very early in the process:
// if the variable is set, we include our page and stop execution after it
function wp_sm_autolog_parse_request( &$wp )
{
    if ( array_key_exists( 'wp_sm_autolog', $wp->query_vars ) ) {
        include( realpath(dirname( __FILE__ ) ) . '/../autolog.php' );
        exit();
    }
}

/** 
  @brief Check if is selected the protected redirect instead of content filtering
  @returns false: if content filtering
           the redirect URL: if redirect protection 
**/
function wp_sm_prot_use_redirect(){
  $redir_option = get_option('sm_prot_redir');
  $redir_option_url = get_option('sm_prot_redir_url');
  
  if(! wp_sm_no_protection() && $redir_option && $redir_option_url && $redir_option_url != ''){
    return $redir_option_url;
  } else {
    return false;
  }
}

/**
  @brief Check if !protection is selected
**/
function wp_sm_no_protection(){
  $redir_option = get_option('sm_prot_redir');
  
  if($redir_option == -1){
    return true;
  } else {
    return false;
  }
}

/**
  @brief Remove protected elements from loops if user has no permisson
**/
add_filter('get_pages', 'wp_sm_remove_nonallowed_pages');
function wp_sm_remove_nonallowed_pages($pages){
  global $user_ID;
  global $userdata;
  wp_get_current_user();
  //die("Current user id" . $userdata);
  if(current_user_can('manage_options') || wp_sm_no_protection()){
    return $pages;
  }

  if (count($pages)>1){

      $pages_ret = array();
      foreach ($pages as $p){
        if(wp_sm_user_has_access($p->ID, 'page', $user_ID)){
          $pages_ret[] = $p;
        }
      }
      return $pages_ret;
  }
  else {
      return $pages;
  }      
}

/**
  @brief Remove protected elements from menus if user has no permission
**/
add_filter( 'wp_get_nav_menu_items', 'wp_sm_exclude_menu_items', null, 3 );
function wp_sm_exclude_menu_items( $items, $menu, $args ) {
  global $user_ID;
  global $userdata;
  wp_get_current_user();
  
  if(current_user_can('manage_options') || wp_sm_no_protection()){
    return $items;
  }
  
  // Iterate over the items to search and destroy
  foreach ( $items as $key => $item ) {
    if ( !wp_sm_user_has_access($item->object_id, 'page', $user_ID) ) unset( $items[$key] );
  }

  return $items;
}


