<?php

/* 
* @brief Returns if the parameters as argument user_login & user_password are correct or not and
*         if the user has admin privilleges
*/
function wp_sm_rest_api_authorize( $args ){
  $creds = array();
  $creds['user_login'] = $args['api_user'];
  $creds['user_password'] = $args['api_key'];
  $creds['remember'] = true;
  
  $user = wp_signon( $creds, false );
  if ( is_wp_error($user) ){
  	return false;
  }
  
  wp_set_current_user($user->ID);
  
  if( !current_user_can('manage_options') ){
    return false;
  }
  
  return true;

}

/*  API FUNCTIONS  */

function wp_sm_rest_api_get_objects( $args ){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }
  
  $types = $args['types'];

  $types_array = explode(',', $types);
  $ret_objs = array();
  
  foreach($types_array as $type){
    
    $ret_objs = array_merge($ret_objs, call_user_func("wp_sm_rest_api_get_".$type));
    
  }
  
  return $ret_objs;
  
}
function wp_sm_rest_api_get_posts() {
    return array();
}

function wp_sm_rest_api_get_pages() {
    $pages = get_pages(array("post_status" => array('publish', 'pending', 'draft', 'future')));
    $res = [];
    foreach ($pages as $page){
      $val = array();
      $val['ID'] = $page->ID;
      $val['post_title'] = $page->post_title;
      $res[] = $val;
    }
    return $res;

}

function wp_sm_rest_api_get_session( $args ){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }
  
  $user_email = $args['user_email'];
  $user_name = $args['user_name'];
  $user_password = $args['user_password'];
  
  $object_id = $args['object_id'];
  $object_class = $args['object_class'];

  // Find the user by email and create the session
  $requesting_user = wp_sm_get_user_by_email($user_email);
  
  if ( !$requesting_user ){
    // Create the user and grant access
    $user_id = wp_sm_create_user($user_email, $user_name, $user_password);
  } else {
    $user_id = $requesting_user->ID;
  }
  if(!wp_sm_user_has_access($object_id, $object_class, $user_id)){
    $permission = wp_sm_grant_access_to_user($user_id, $object_id, $object_class);
  }
  
  $generated_session_id = uniqid();
  update_user_meta($user_id, 'wp_sm_session_id', $generated_session_id);
  
  if(!wp_sm_object_is_protected($object_id, $object_class)){
    // Only for pages
    update_post_meta($object_id, "wp_sm_protected", "1");
  }
  
  return array("session_id" => $generated_session_id);    
}

function wp_sm_rest_api_create_user_with_access($args){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }
  
  $user_email = $args['user_email'];
  $user_name = $args['user_name'];
  $user_password = $args['user_password'];
  
  $object_id = $args['object_id'];
  $object_class = $args['object_class'];
  
  $user_id = wp_sm_create_user($user_email, $user_name, $user_password);
  
  if(!wp_sm_user_has_access($object_id, $object_class, $user_id)){
    $permission = wp_sm_grant_access_to_user($user_id, $object_id, $object_class);
  }
  
  return array("result" => "ok");
}

function wp_sm_rest_api_disable_access_to_user($args){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }
  
  $user_email = $args['user_email'];
  //$user_name = $args['user_name'];
  //$user_password = $args['user_password'];
  
  $object_id = $args['object_id'];
  $object_class = $args['object_class'];
  
  $user_id = wp_sm_create_user($user_email, $user_name, $user_password);
  if(wp_sm_user_has_access($object_id, $object_class, $user_id)){
    $permission = wp_sm_disable_access_to_user($user_id, $object_id, $object_class);
  }
  
  return array("result" => "ok");
}

function wp_sm_rest_api_ping( $args ){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }

  return array("result" => "ok");    
}

function wp_sm_rest_api_protect_object($args){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }

  $object_id = $args['object_id'];
  $object_class = $args['object_class'];
  
  $protect_childs = $args['protect_childs'];
  
  update_post_meta($object_id, "wp_sm_protected", "1");
  
  if($protect_childs && $protect_childs !== 'false'){
    update_post_meta($object_id, "wp_sm_protect_childs", "1");
  } else {
    delete_post_meta($object_id, "wp_sm_protect_childs");
  }
  
  return array("result" => "ok");
}

function wp_sm_rest_api_unprotect_object($args){
  if( !wp_sm_rest_api_authorize($args) ){ return array("error" => "Invalid credentials"); }

  $object_id = $args['object_id'];
  $object_class = $args['object_class'];
  
  delete_post_meta($object_id, "wp_sm_protected");
  delete_post_meta($object_id, "wp_sm_protect_childs");
  
  return array("result" => "ok");
}

function wp_sm_rest_api_public_ping( $args ){
  return array("result" => "ok");    
}


/*   END API FUNCTIONS   */

/* ---------------------------------------------------------------------------------------- */

/*   "MAIN" METHOD    */

if( isset($_REQUEST['wp_sm_action']) && function_exists("wp_sm_rest_api_".$_REQUEST['wp_sm_action'])){
  echo json_encode(  call_user_func("wp_sm_rest_api_".$_REQUEST['wp_sm_action'], $_REQUEST)  );
}
exit;

