<?php
/*
Plugin Name: WP SnakeMember
Plugin URI: http://snakemember.com/
Description: Wordpress integration plugin for SnakeMember
Author: Michele Cumer
Author URI: http://www.snakemember.com
Version: 2.0.5
*/

// WP Activerecord
define("WP_AR_PREFIX", $wpdb->prefix);
require_once realpath(dirname(__FILE__)) . '/lib/ActiveRecordLib/ActiveRecord.php';
require_once realpath(dirname(__FILE__)) . '/wp_ar_functions/functions.php';
// Tracking
require_once realpath(dirname(__FILE__)) . "/affiliate_tracking/affiliate_tracking.php";
// Campaigns & subscription widget
require_once realpath(dirname(__FILE__)) . "/campaigns/campaigns.php";
// Protected Pages
require_once realpath(dirname(__FILE__)) . "/xmlrpc/functions.php";
require_once realpath(dirname(__FILE__)) . "/permissions/functions_permissions.php";
// Admin area
require_once realpath(dirname(__FILE__)) . "/area_adm/view.php";
require_once realpath(dirname(__FILE__)) . "/area_adm/ajax.php";
// Secure downloads
require_once realpath(dirname(__FILE__)) . "/secure_downloads/functions.php";

// OTOs
require_once realpath(dirname(__FILE__)) . "/otos/functions.php";

// REST API
require_once realpath(dirname(__FILE__)) . "/rest/sm_api.php";

// WooCommerce functions
require_once realpath(dirname(__FILE__)) . "/woocommerce_functions/snake_subscribe_curl_function.php";
require_once realpath(dirname(__FILE__)) . "/woocommerce_functions/functions.php";

// Shortcodes functions
require_once realpath(dirname(__FILE__)) . "/shortcodes/shortcodes.php";

// WebForm widget
require_once realpath(dirname(__FILE__)) . "/webforms/widget.php";

function sm_wp_activerecord_init(){
    ActiveRecord\Config::initialize(function($cfg)
    {
      $cfg->set_connections(array('development' => 'mysql://'.constant("DB_USER").':'.constant("DB_PASSWORD").'@'.constant("DB_HOST").'/' . constant("DB_NAME")));
    });
}

add_action("init", "sm_wp_activerecord_init", 5);
sm_wp_activerecord_init();

/* END - LOAD ActiveRecord */

load_plugin_textdomain('wp-sm', false, dirname( plugin_basename( __FILE__ ) ) . '/langs');

global $_site_protocol, $snakemember_domain, $wp_sm_db_version;

$test = false;

if($test){
  $_site_protocol = 'http://';
  $snakemember_domain = "test.areamembri.it";
} else {
  $_site_protocol = 'https://';
  $snakemember_domain = "areamembri.it";
}
$wp_sm_db_version = 1.0;

define("SNAKEMEMBER_URL", $_site_protocol . $snakemember_domain);

#Tracking
add_shortcode('aff_link', 'sm_aff_link');
add_action('wp_footer', 'track_click_js');

#Campaigns
add_action( 'wp_ajax_get_snake_lists', 'get_snake_lists_callback' );


function sm_wp_frontend_style(){
  wp_enqueue_style("sm_webforms", plugins_url( "style.css", __FILE__ ) );
}


function sm_wp_admin_style() {
    wp_enqueue_style('sm_wp_admin_style', plugins_url('wp-admin.css', __FILE__));
}
add_action('admin_enqueue_scripts', 'sm_wp_admin_style');
add_action('wp_enqueue_scripts', 'sm_wp_frontend_style');


# XML-RPC Methods
function sm_wp_xmlrpc_methods( $methods ) {
    $methods['SM.get_objects'] = 'sm_wp_get_objects';
    $methods['SM.get_session'] = 'sm_wp_get_session';
    $methods['SM.create_user_with_access'] = 'sm_wp_create_user_with_access';
    $methods['SM.disable_access_to_user'] = 'sm_wp_disable_access_to_user';
    $methods['SM.protect_object'] = 'sm_wp_protect_object';
    $methods['SM.unprotect_object'] = 'sm_wp_unprotect_object';
    $methods['SM.ping'] = 'sm_wp_ping';
    return $methods;   
}
add_filter( 'xmlrpc_methods', 'sm_wp_xmlrpc_methods');
register_activation_hook(__FILE__, 'sm_wp_db_tables');

# Page view methods
add_filter( 'manage_pages_columns', 'wp_sm_modify_pages_table' );
add_action( 'manage_pages_custom_column', 'wp_sm_modify_pages_table_row', 10, 2 );

add_filter('the_content', 'wp_sm_filter_protected_pages');
add_filter('template_redirect', 'wp_sm_filter_protected_pages_redirect', 1);

# Register "wp_sm_autolog" URL
add_action( 'init', 'wp_sm_autolog_init', 12 );
add_action( 'query_vars', 'wp_sm_autolog_query_vars' );
add_action( 'parse_request', 'wp_sm_autolog_parse_request' );

# Register "wp_sm_secure_download" URL
add_action( 'init', 'wp_sm_secure_downloads_init', 12 );
add_action( 'query_vars', 'wp_sm_secure_downloads_query_vars' );
add_action( 'parse_request', 'wp_sm_secure_downloads_parse_request' );

# Register "wp_sm_rest_api" URL
add_action( 'init', 'wp_sm_rest_api_init', 12 );
add_action( 'query_vars', 'wp_sm_rest_api_query_vars' );
add_action( 'parse_request', 'wp_sm_rest_api_parse_request' );

# Affiliate ID Management
add_action('init', 'track_affiliate', 12);

function wp_sm_flush_rewrite() {
  wp_sm_autolog_init();
  wp_sm_secure_downloads_init();
  wp_sm_rest_api_init();
	flush_rewrite_rules();
}

register_activation_hook( __FILE__, 'wp_sm_flush_rewrite' );

# Protected download tags
add_shortcode( 's3_secure_download', 'wp_sm_secure_s3_link' );
#add_shortcode( 'sm_s3_secure_video', 'wp_sm_secure_s3_video' );


